/**
 * The contents of this file are subject to the license and copyright detailed in the LICENSE and NOTICE files at the root of the source tree and available online at
 * http://www.askosi.org/license.html
 */
package be.destin.skos.core;

import be.destin.rdf.changes.Status;

/**
 * @author Christophe Dupriez
 *
 * This class enumerates the possible format for a label, a note...
 * It can be:
 * REGULAR: just plain Unicode text
 * HTML: the text contains HTML tags and may even contains sur-encoded entities (something like &amp;amp; meaning "&")
 *     The indexing/searching of ASKOSI will have to be adapted to HTML format
 * URL: the text is located at the provided URL. No provision exist for now to get this text and index it for search...
 *     Meanwhile it is rendered with something like <a href="URL" target="_blank"><img src="icon for outside link"/></a> 
 */
public enum TermFormat { REGULAR, HTML, URL, REL;

/**
 * @return all values as a comma separated String
 */
public static String list() {
	StringBuffer result = new StringBuffer();
	Status[] types = Status.values();
	for (int i=0; i<types.length; i++) {
		Status aType = types[i];
		result.append(aType.toString());
		if ( (i+1) < types.length) result.append(", ");
	}
	return result.toString();
}

}
