/**
 * The contents of this file are subject to the license and copyright detailed in the LICENSE and NOTICE files at the root of the source tree and available online at
 * http://www.askosi.org/license.html
 */
package be.destin.skos.view;

import java.util.Collection;
import java.util.LinkedList;
import java.util.TreeMap;

import javax.xml.bind.annotation.XmlAttribute;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlID;
import javax.xml.bind.annotation.XmlTransient;
import javax.xml.bind.annotation.XmlType;

import be.destin.skos.core.Concept;
import be.destin.skos.core.ReferringApplicationList;
import be.destin.skos.core.SchemeInterface;
import be.destin.skos.core.SkosManager;

/**
 * @author Christophe Dupriez
 * 
 */
@XmlType(propOrder={"prefLabel","usage","narrowerUsage","broader","narrower"})
public class ViewConcept {

	private Concept base;
	private SkosView view;
	private int depth = 1; // TOP by default

	public ViewConcept () {
		super();
	}

	public ViewConcept (SkosView aList, Concept anEntry, int depth) {
		super();
		this.view = aList;
		this.base = anEntry;
		this.depth = depth;
	}
	/**
	 * @return the code
	 */
	@XmlID
	@XmlAttribute(required = true)
	public String getAbout() {
		return base.getAbout();
	}

	/**
	 * @return the parents
	 */
	@XmlElement
	public LinkedList<String> getBroader() {
		return base.getAboutBroader();
	}
	
	/**
	 * @return the base
	 */
	@XmlTransient
	public Concept getConcept() {
		return this.base;
	}

	/**
	 * @return the children
	 */
	@XmlElement
	public LinkedList<ViewConcept> getNarrower() {
		if (base==null) return null;
		SkosManager skosManager = view.getSkosManager();
		if (skosManager == null) return null;
		if (view.getType() != SkosViewType.TREE) return null; // Only trees have a depth...
		if (depth >= Concept.maxDepth) return null; // Deeper is not possible!
		TreeMap<String,ViewConcept> sorter = new TreeMap<String,ViewConcept>();
		SchemeInterface aList = skosManager.getScheme(view.getAbout());
		Collection<String> baseChildren = this.base.getAboutNarrower();
		if (baseChildren==null) return null;
		view.setSomeRejects(false);
		for (String aChild : baseChildren) {
			if (view.filteredEntries == null || view.filteredEntries.contains(aChild)) {
				Concept anEntry = aList.getConcept(aChild);
				if (anEntry != null) {
					sorter.put(anEntry.getSortLabel(view.getLang()),new ViewConcept (view, anEntry ,depth+1) );
				}
			} else view.setSomeRejects(true);
		}
		LinkedList<ViewConcept> children = new LinkedList<ViewConcept>();
		for (ViewConcept aViewConcept : sorter.values()) {
			children.add(aViewConcept);
		}
		return children;
	}

	/**
	 * @return all the aggregatedChildrenUsages
	 */
	@XmlElement
	public ReferringApplicationList getNarrowerUsage() {
		return this.base.getNarrowerUsage();
	}

	/**
	 * @return the names
	 */
	@XmlElement
	public String getPrefLabel() {
		return this.base.getPrefLabel(view.getLang());
	}

	/**
	 * @return the usages
	 */
	@XmlElement
	public ReferringApplicationList getUsage() {
		return this.base.getUsage();
	}

	/**
	 * @param base the base to set
	 */
	public void setConcept(Concept base) {
		this.base = base;
	}

	/**
	 * @param aggregatedChildrenUsages
	 *            the aggregatedChildrenUsages to set
	 */
	public void setNarrowerUsage(
			ReferringApplicationList aggregatedChildrenUsages) {
		this.base.setNarrowerUsage(aggregatedChildrenUsages);
	}

	public String toString() {
		String aName = this.getPrefLabel();
		if ( aName == null) aName = "no name";
		return "[" + getAbout() + "=" + aName + "]";
	}

	/**
	 * @return the skosManager
	 */
	@XmlTransient
	public SkosManager getSkosManager() {
		return view.getSkosManager();
	}

}
