<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE stylesheet [
<!ENTITY mesh  "http://www.nlm.nih.gov/mesh/2006#" >
]>
<xsl:stylesheet version="1.0"
	xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:dc="http://purl.org/dc/elements/1.1/"
	xmlns:dcterms="http://purl.org/dc/terms/" xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"
	xmlns:mesh="http://www.nlm.nih.gov/mesh/2006#" xmlns:skos="http://www.w3.org/2004/02/skos/core#">
	<!--
		Developed by Christophe Dupriez for the ASKOSI project
		http://www.askosi.org Please feel free to modify but please keep this
		reference
	-->
	<xsl:output method="xml" version="1.0" omit-xml-declaration="no"
		indent="no" encoding="utf-8" />

	<xsl:template match="text()">
		<!--
			Do nothing. Override, effectively, the "Built-In" rule which will
			process all text inside elements otherwise not matched by any
			xsl:template. Note: With this in place, be sure to then provide
			templates or "value-of" statements to actually _get_ the (desired)
			text out to the result document!
		-->
	</xsl:template>

	<xsl:template match="/rdf:RDF">
		<xsl:element name="conceptScheme">
			<xsl:attribute name="about">mesh</xsl:attribute>
			<xsl:element name="title">
				<xsl:attribute name="lang">en</xsl:attribute>
				Medical Subject Headings
			</xsl:element>
			<xsl:element name="icon">
				<xsl:attribute name="lang">en</xsl:attribute>
				[root]/image/mesh.png
			</xsl:element>
			<xsl:for-each select="rdf:Description[starts-with(@rdf:about,'&mesh;D')]">
				<xsl:call-template name="oneConcept" />
			</xsl:for-each>
		</xsl:element>
	</xsl:template>

	<xsl:template name="oneConcept">
		<xsl:element name="concept">
			<xsl:attribute name="about"><xsl:value-of
				select="substring-after(@rdf:about,'#')" /></xsl:attribute>
			<xsl:variable name="about" select="@rdf:about" />
			<xsl:for-each select="@skos:prefLabel">
				<xsl:element name="prefLabel">
					<xsl:attribute name="lang">en</xsl:attribute>
					<xsl:value-of select="." />
				</xsl:element>
			</xsl:for-each>
			<xsl:for-each select="./skos:prefLabel">
				<xsl:element name="prefLabel">
					<xsl:attribute name="lang">en</xsl:attribute>
					<xsl:value-of select="." />
				</xsl:element>
			</xsl:for-each>
			<xsl:for-each select="@skos:altLabel">
				<xsl:element name="altLabel">
					<xsl:attribute name="lang">en</xsl:attribute>
					<xsl:value-of select="." />
				</xsl:element>
			</xsl:for-each>
			<xsl:for-each select="./skos:altLabel">
				<xsl:element name="altLabel">
					<xsl:attribute name="lang">en</xsl:attribute>
					<xsl:value-of select="." />
				</xsl:element>
			</xsl:for-each>
			<xsl:for-each select="@skos:hiddenLabel">
				<xsl:element name="hiddenLabel">
					<xsl:attribute name="lang">en</xsl:attribute>
					<xsl:value-of select="." />
				</xsl:element>
			</xsl:for-each>
			<xsl:for-each select="./skos:hiddenLabel">
				<xsl:element name="hiddenLabel">
					<xsl:attribute name="lang">en</xsl:attribute>
					<xsl:value-of select="." />
				</xsl:element>
			</xsl:for-each>
			<xsl:for-each select="./skos:changeNote">
				<xsl:element name="changeNote">
					<xsl:attribute name="lang">en</xsl:attribute>
					<xsl:value-of select="." />
				</xsl:element>
			</xsl:for-each>
			<xsl:for-each select="./skos:definition">
				<xsl:element name="definition">
					<xsl:attribute name="lang">en</xsl:attribute>
					<xsl:value-of select="." />
				</xsl:element>
			</xsl:for-each>
			<xsl:for-each select="./skos:editorialNote">
				<xsl:element name="editorialNote">
					<xsl:attribute name="lang">en</xsl:attribute>
					<xsl:value-of select="." />
				</xsl:element>
			</xsl:for-each>
			<xsl:for-each select="./skos:example">
				<xsl:element name="example">
					<xsl:attribute name="lang">en</xsl:attribute>
					<xsl:value-of select="." />
				</xsl:element>
			</xsl:for-each>
			<xsl:for-each select="./skos:historyNote">
				<xsl:element name="historyNote">
					<xsl:attribute name="lang">en</xsl:attribute>
					<xsl:value-of select="." />
				</xsl:element>
			</xsl:for-each>
			<xsl:for-each select="./skos:scopeNote">
				<xsl:element name="scopeNote">
					<xsl:attribute name="lang">en</xsl:attribute>
					<xsl:value-of select="." />
				</xsl:element>
			</xsl:for-each>
			<xsl:for-each select="./skos:exactMatch">
				<xsl:element name="exactMatch">
					<xsl:value-of select="substring-after(@rdf:resource,'#')" />
				</xsl:element>
			</xsl:for-each>
			<xsl:for-each select="./skos:closeMatch">
				<xsl:element name="closeMatch">
					<xsl:value-of select="substring-after(@rdf:resource,'#')" />
				</xsl:element>
			</xsl:for-each>
			<xsl:for-each select="./skos:broader">
				<xsl:element name="broader">
					<xsl:value-of select="substring-after(@rdf:resource,'#')" />
				</xsl:element>
			</xsl:for-each>
			<xsl:for-each select="/rdf:RDF/skos:Concept[@rdf:about=$about]/skos:broader">
				<xsl:element name="broader">
					<xsl:value-of select="substring-after(@rdf:resource,'#')" />
				</xsl:element>
			</xsl:for-each>
			<xsl:for-each select="./skos:narrower">
				<xsl:element name="narrower">
					<xsl:value-of select="substring-after(@rdf:resource,'#')" />
				</xsl:element>
			</xsl:for-each>
			<xsl:for-each select="./skos:related">
				<xsl:element name="related">
					<xsl:value-of select="substring-after(@rdf:resource,'#')" />
				</xsl:element>
			</xsl:for-each>
			<xsl:for-each select="./skos:broadMatch">
				<xsl:element name="broadMatch">
					<xsl:value-of select="substring-after(@rdf:resource,'#')" />
				</xsl:element>
			</xsl:for-each>
			<xsl:for-each select="./skos:narrowMatch">
				<xsl:element name="narrowMatch">
					<xsl:value-of select="substring-after(@rdf:resource,'#')" />
				</xsl:element>
			</xsl:for-each>
			<xsl:for-each select="./skos:relatedMatch">
				<xsl:element name="relatedMatch">
					<xsl:value-of select="substring-after(@rdf:resource,'#')" />
				</xsl:element>
			</xsl:for-each>
		</xsl:element>
	</xsl:template>
</xsl:stylesheet>
